/*
** IntuiDemo_2.c: Beispiel zur Programmierung von pOS-Gadgets, pOS-Mens
**                und der Nachrichten-Verarbeitung
**
** 12.03.-03.05.1997, Michael Christoph
**
** Shell-Fail-Code nach dem Programmlauf:
**  DOSFAIL_OK    => alles ok
**  DOSFAIL_WARN  => bergebene Argumente waren falsch
**  DOSFAIL_ABORT => Benutzer hat abgebrochen
**  DOSFAIL_ERROR => (unbenutzt)
**  DOSFAIL_FAIL  => Fehler in der Programmumgebung (z.B. Speichermangel)
**
** bersetzen mit MaxonC:
**  mcpp:mcppc3 -gs -o IntuiDemo_2 p:pLib/StartCode.o IntuiDemo_2.c -l pOSStub -l pOS -i p: -j plib:
** bersetzen mit StormC:
**  stormc:stormsys/stormc IntuiDemo_2.c -i p:
**  stormc:stormsys/stormlink p:pLib/StartCode.o IntuiDemo_2.c LIB pOSStub LIB pOS LIBPATH plib: LIB stormc:lib/strom.lib TO IntuiDemo_2
** bersetzen mit SAS-C
**  sc NOLINK DATA=FAR CODE=FAR NOSTACKCHECK INCLUDEDIR=p: OBJECTNAME IntuiDemo_3.o IntuiDemo_3.c
**     bei Bedarf 'IGNORE=79,110' mit angeben
**  slink p:pLib/StartCode.o IntuiDemo_3.o to IntuiDemo_3 LIB plib:pOSStub.lib plib:pOS.lib SMALLDATA SMALLCODE
**     bei Bedarf 'LIB sc.lib' mit angeben
**
** p:    ist ein (Multi-)Assign auf das pOS-Include-Verzeichnis
** plib: ist ein (Multi-)Assign auf das pOSLib-Verzeichnis
*/

#define PROGNAME "IntuiDemo"
#define VERSIONR "1.2"
#define PROGDATE "03.05.1997"  /* ersetzbar durch __DATE__ oder __DATE2__ oder __AMIGADATE__ */
#define COPYRIGT "Copyright  Mr.1997 by Michael Christoph"

/******************************* INCLUDES *************************************/

#include <pExec/Types.h>
#include <pDOS/ArgTags.h>
#include <pDOS/DosSig.h>
#include <pIntui/Beep.h>
#include <pIntui/FraClass.h>
#include <pIntui/IntuMsg.h>
#include <pIntui/IType.h>
#include <pIntui/OClass.h>
#include <pIntui/Tags.h>
#include <pGadget/GadItem.h>
#include <pGadget/PropGad.h>
#include <pScreen/Screen.h>
#include <pScreen/ScrTags.h>
#include <pScreen/Window.h>
#include <pGFX/RastPort.h>

#include <proto/pExec2.h>
#include <proto/pDOS2.h>
#include <proto/pIntui2.h>
#include <proto/pGFX2.h>
#include <proto/pUtil2.h>

/************************ VARIABLEN DEKLARATIONEN *****************************/

const CHAR *gb_PrgHelpText="$H(PUBSCREEN/K)Name of the pubscreen we visite";
const CHAR *gb_PrgHeader  =PROGNAME" - Example with Gadgets, Mens, Message-Handling";
const CHAR *gb_PrgVersion ="$VER: "PROGNAME" "VERSIONR" ("PROGDATE") ("COPYRIGT")";
const CHAR *gb_PrgTemplate="PUBSCREEN/K";

enum { ARG_pubscreen, ARG_MAX };
ULONG args[ARG_MAX]={ 0 };

/************************ VARIABLEN DEKLARATIONEN *****************************/

struct pOS_IntuiDevice *gb_IntuiBase;
struct pOS_GfxBase     *gb_GfxBase;
struct pOS_UtilityBase *gb_UtilityBase;

/************************ VARIABLEN DEKLARATIONEN *****************************/

/* Zeiger auf die einzelnen Gadgets */
struct pOS_Gadget *gb_CheckboxGad;
struct pOS_Gadget *gb_UnvisibleGad;
struct pOS_Gadget *gb_ButtonGad;
struct pOS_Gadget *gb_StringGad;

struct pOS_Gadget *gb_PopupGad;
struct pOS_Gadget *gb_SliderGad;
struct pOS_Gadget *gb_TextGad;     /* berschrift fr MXGad */
struct pOS_Gadget *gb_MXGad;
struct pOS_Gadget *gb_ListviewGad;

struct pOS_Gadget *gb_InfotextGad;

/* the big gadget: dieses Gadget enthlt alle einzelnen Gadgets gruppiert */
/*                 und wird im Fenster eingehngt */
struct pOS_Gadget *gb_AllGadgetGrp;

enum GadgetMask
{
  GM_None      =0x0000,
  GM_Checkbox  =0x0001,
  GM_Unvisible =0x0002,
  GM_Button    =0x0004,
  GM_String    =0x0008,
  GM_Popup     =0x0010,
  GM_Slider    =0x0020,
  GM_Text      =0x0040,
  GM_MX        =0x0080,
  GM_Listview  =0x0100,
  GM_Infotext  =0x0200,
  GM_Bar       =0x0400,  /* unused */
};


/* Namen sind spter verstndlicher als die reinen Zahlenwerte */
enum JahresZeit
{
  JZ_Fruehling = 0,
  JZ_Sommer    = 1,
  JZ_Herbst    = 2,
  JZ_Winter    = 3,
  JZ_MaxEntry
};

/* Eintrge fr das ListView-Gadget, Abgeschlossen mit NULL-Name */
const struct pOS_GadgetTagItem gb_ListviewEntry[]={
  { "Frhling",JZ_Fruehling },
  { "Sommer",  JZ_Sommer },
  { "Herbst",  JZ_Herbst },
  { "Winter",  JZ_Winter },
  NULL
};

/* Eintrge fr das Popup-Gadget, Abgeschlossen mit NULL-Name */
const struct pOS_GadgetTagItem gb_PopupEntry[]={
  { "Frhling",JZ_Fruehling },
  { "Sommer",  JZ_Sommer },
  { "Herbst",  JZ_Herbst },
  { "Winter",  JZ_Winter },
  NULL
};

/* Eintrge fr das MutualExlude-Gadget, Abgeschlossen mit NULL-Zeiger */
const CHAR *gb_MXEntry[]={
  "Frhling",
  "Sommer",
  "Herbst",
  "Winter",
  NULL
};


/* enthlt immer die aktive Jahreszeit (enum JahresZeit) */
/* da mit dieser Variable auch gerechnet wird, darf sie nicht */
/* als 'enum JahresZeit' deklariert werden. */
UBYTE gb_AktiveJahreszeit=JZ_Fruehling;

/****************************** PRTOTYPEN *************************************/

BOOL CreateAllGadgets(const struct pOS_DrawInfo *dri);
void SetDatas(struct pOS_Window *win, UBYTE jz, UWORD gm);
SLONG MessageLoop(struct pOS_Window *win);

dossize_t pOS_Printf(const CHAR *format, ...)
{
  return( pOS_VFWritef(pOS_GetStdOutput(),format,(ULONG*)(&format)+1) );
}

/******************************************************************************/

BOOL CreateAllGadgets(const struct pOS_DrawInfo *dri)
{
  /*
  ** Returned TRUE wenn alle Gadgets erzeugt wurden,
  **          sonst FALSE. Dann sind bereits alle Gadgets
  **          wieder disposed (freigegeben).
  */

  /* ein Checkbox-Gadget */
  gb_CheckboxGad = (struct pOS_Gadget *) pOS_NewIObject(NULL,
    dri->dri_Names[SCRNAM_GCheckBoxClass],0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_StdLableName,      "_Checkbox",
    /*
    ** Inline-Hilfetexte werden durch ein "#" als erstes
    ** Zeichen eingeleitet. Mehrzeilig ber "*n" mglich.
    */
    ICLTAG_HelpID,            "#Checkbox - Gadget",
    /*
    ** bei externer Hilfetext-Datei z.B.
    ** ICLTAG_HelpID,            "CheckboxGad",
    */
    ICLTAG_Selected,          TRUE,         /* eingedrckt starten */
    TAG_DONE);

  /* ein unsichtbares Gadget */
  /* Trick: dehnt sich spter aus und drckt den Button an den rechten Rand */
  gb_UnvisibleGad = (struct pOS_Gadget *) pOS_NewIObject(NULL,
    dri->dri_Names[SCRNAM_GClass],0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_Gwk,               1,
    TAG_DONE);

  /* ein normaler Button */
  gb_ButtonGad = (struct pOS_Gadget *) pOS_NewIObject(NULL,
    dri->dri_Names[SCRNAM_GButtonClass],0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_RelVerify,         TRUE,
    ICLTAG_AddHeight,         8,            /* mehr Hhe geben */
    ICLTAG_HelpID,            "#Button - Gadget",
    IOBJTAG_StaticString,     " _Button ",  /* links und rechts mehr Luft lassen (alternativ AddWidth mglich) */
    TAG_DONE);

  /* ein String-Gadget; Inhalt ist drag&dropfhig */
  gb_StringGad = (struct pOS_Gadget *) pOS_NewIObject(NULL,
    dri->dri_Names[SCRNAM_GStrClass],0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_RelVerify,         TRUE,
    ICLTAG_StdLableName,      "_String",
    ICLTAG_HelpID,            "#String - Gadget",
    STRGADTAG_StdDropAtt,     TRUE,
    STRGADTAG_StdDragAtt,     TRUE,
    STRGADTAG_MaxChars,       100,      /* Anzahl Zeichen ohne Nullbyte */
    STRGADTAG_String,         gb_MXEntry[gb_AktiveJahreszeit],
    TAG_DONE);

/* ----------------------------------------------------- */

  /* ein Popup-Gadget */
  gb_PopupGad = (struct pOS_Gadget *) pOS_NewIObject(NULL,
    dri->dri_Names[SCRNAM_GLVPopupClass],0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_Immediate,         TRUE,
    ICLTAG_RelVerify,         TRUE,
    ICLTAG_UpdateGadget,      TRUE,
    ICLTAG_StdLableName,      "_Popup",
    ICLTAG_HelpID,            "#Popup - Gadget",
    LVGADTAG_GadgetTagItem,   gb_PopupEntry,
    LVGADTAG_ItemID,          gb_AktiveJahreszeit,
    LVGADTAG_AlwaysOne,       TRUE,             /* es mu immer einer ausgewhlt sein */
    LVGADTAG_MinStdHeight,    4*pOS_DRISTDFAC,  /* geffnet vier Zeilen sichtbar */
    POPUPGADTAG_MinStdWidth,  8*pOS_DRISTDFAC,  /* Mindesbreite Popup-Gadget zugeklappt; grob gesagt 8 Zeichen */
    POPUPGADTAG_MinPopStdWidth,8*pOS_DRISTDFAC, /* Mindesbreite Popup-Gadget aufgeklappt; grob gesagt 8 Zeichen */
    POPUPGADTAG_LayoutMode,   IOBLAY_Bottom,    /* Popup nach oben ffnen */
    TAG_DONE);

  /* ein Slider-Gadget */
  gb_SliderGad = (struct pOS_Gadget *) pOS_NewIObject( NULL,
    dri->dri_Names[SCRNAM_GPropClass], 0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_UpdateGadget,      TRUE,
    ICLTAG_RelVerify,         TRUE,
    ICLTAG_StdLableName,      "Sli_der",
    ICLTAG_HelpID,            "#Slider - Gadget",
    PRGADTAG_Flags,           PRGADF_FreeHoriz, /* horizontal verschiebbar */
    PRGADTAG_CreateUDGads,    TRUE,             /* UpDown Pfeil erzeugen */
    PRGADTAG_Total,           4,                /* Anzahl Eintrge */
    PRGADTAG_Visible,         1,
    PRGADTAG_Top,             gb_AktiveJahreszeit,  /* aktiver beim ffnen */
    PRGADTAG_CrtTxtObj,       TRUE,             /* autom. Textanzeigeobjekt erstellen und setzen */
    PRGADTAG_TxtArray,        gb_MXEntry,       /* selbe Struktur; vom MX mibraucht */
    TAG_DONE);

  /* ein einfacher Text ohne Rahmen, als berschrift fr MX-Gruppe */
  /* es wird der selbe Hilfetext, wie bei der MX-Gruppe angezeigt */
  gb_TextGad = (struct pOS_Gadget *) pOS_NewIObject(NULL,
    dri->dri_Names[SCRNAM_GTxtBoxClass], 0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_FrameType,         IFRACLTYP_EraseBg,/* Hintergrund nicht beeinflussen */
    ICLTAG_GadBorderType,     GADBORTYP_None,   /* ohne Rahmen */
    ICLTAG_HelpID,            "#MutualExclude - Gadget",
    IOBJTAG_StaticString,     "_MutualExclude",
    IOBJTAG_Underscore,       '_',              /* Unterstich ist ein Shortkey */
    TAG_DONE);

  /* ein MX-Gadget */
  gb_MXGad = (struct pOS_Gadget *) pOS_NewIObject(NULL,
    dri->dri_Names[SCRNAM_GMXClass],0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_Gwk,               1,
    ICLTAG_HelpID,            "#MutualExclude - Gadget",
    MXGADTAG_StrLabels,       gb_MXEntry,
    MXGADTAG_Active,          gb_AktiveJahreszeit,
    TAG_DONE);

  /* ein ListView */
  gb_ListviewGad = (struct pOS_Gadget *) pOS_NewIObject(NULL,
    dri->dri_Names[SCRNAM_GLVClass],0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_Gwk,               2,
    ICLTAG_Immediate,         TRUE,
    ICLTAG_RelVerify,         TRUE,
    ICLTAG_UpdateGadget,      TRUE,
    ICLTAG_StdLableName,      "_Listview",
    ICLTAG_HelpID,            "#Listview - Gadget",
    ICLTAG_LableLayMode,      IOBLAY_Top, /* Label oberhalb platzieren */
    LVGADTAG_VAutoGad,        TRUE,       /* rechts einen Schieber erzeugen */
    LVGADTAG_CursorScroll,    TRUE,       /* Cursortasten zum Inhalt verschieben mglich */
    LVGADTAG_CursorSelect,    TRUE,       /* Cursortasten zum Auswhlen mglich */
    LVGADTAG_AlwaysOne,       TRUE,       /* es mu immer einer selektiert sein */
    LVGADTAG_CR,              TRUE,       /* Return als Besttigung mglich */
    LVGADTAG_GadgetTagItem,   gb_ListviewEntry,
    LVGADTAG_ItemID,          gb_AktiveJahreszeit,
    TAG_DONE);

/* ----------------------------------------------------- */

  /* ein Infofeld */
  gb_InfotextGad = (struct pOS_Gadget *) pOS_NewIObject(NULL,
    dri->dri_Names[SCRNAM_GTxtBoxClass],0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_HelpID,            "#Txtbox - Gadget",
    IOBJTAG_StaticString,     "Infofeld",      /* mu mind. ein Leerstring sein beim ffnen */
    BUTGADTAG_MinStdWidth,    8*pOS_DRISTDFAC, /* etwa 8 Zeichen breit */
    BUTGADTAG_MinStdHeight,   1*pOS_DRISTDFAC, /* und 1 Zeile hoch */
    TAG_DONE);

  /* Beispiel fr eine Trennlinie
  ** gb_BarGad = (struct pOS_Gadget *) pOS_NewIObject(NULL,
  **   dri->dri_Names[SCRNAM_GBarClass], 0,
  **   ICLTAG_DrawInfo,          dri,
  **   TAG_DONE);
  */

/* ----------------------------------------------------- */

  /*
  ** alle Gadgets zu einer groen Gruppe verbinden.
  ** Alternativ knnen auch kleine temporre Gruppen erzeugt werden
  ** und diese erst in der groen zusammengesetzt werden.
  */

  gb_AllGadgetGrp = (struct pOS_Gadget *) pOS_NewIObject( NULL,
    dri->dri_Names[SCRNAM_GGroupClass], 0,
    ICLTAG_DrawInfo,          dri,
    ICLTAG_AutoDelete,        TRUE,  /* alle Members automatisch disposen */
    ICLTAG_Gwk,               4,
    GRPGADTAG_SpaceVert,      4,     /* vertikaler Abstand zwischen den Gadgets/Gruppen */
    GRPGADTAG_BorLeft,        4,
    GRPGADTAG_BorTop,         4,
    GRPGADTAG_BorRight,       4,
    GRPGADTAG_BorBottom,      4,

    GRPGADTAG_TstAddGadget,   pOS_NewIObject( NULL,
        dri->dri_Names[SCRNAM_GGroupClass], 0,
        ICLTAG_DrawInfo,          dri,
        ICLTAG_GadBorderType,     GADBORTYP_XBox,
        /*
        ** Soll der Hinterund innerhalb der Gruppe nicht angezeigt
        ** werden, erreichen Sie das, indem der Zeichenmodus auf
        ** DRMD_Jam2 gesetzt wird:
        **  IOBJTAG_DrMd,    DRMD_Jam2,
        ** Optinal knnen Sie noch die Hintergrundfarbe definieren:
        **  IOBJTAG_BGPen,   dri->dri_Pens[SCRPEN_Background],
        */
        GRPGADTAG_BorLeft,        4,
        GRPGADTAG_BorTop,         4,
        GRPGADTAG_BorRight,       4,
        GRPGADTAG_BorBottom,      4,
        GRPGADTAG_TstAddGadget,   pOS_NewIObject( NULL,
            dri->dri_Names[SCRNAM_GGroupClass], 0,
            ICLTAG_DrawInfo,          dri,
            GRPGADTAG_Horiz,          TRUE,
            GRPGADTAG_TstAddGadget,   gb_CheckboxGad,
            GRPGADTAG_TstAddGadget,   gb_UnvisibleGad,
            GRPGADTAG_TstAddGadget,   gb_ButtonGad,
            TAG_DONE),
        GRPGADTAG_TstAddGadget,   gb_StringGad,
        TAG_DONE),

    GRPGADTAG_TstAddGadget,   pOS_NewIObject( NULL,
        dri->dri_Names[SCRNAM_GGroupClass], 0,
        ICLTAG_DrawInfo,          dri,
        ICLTAG_Gwk,               2,
        ICLTAG_GadBorderType,     GADBORTYP_XBox,
        /*
        ** Hintergrund immer leer zeichen; siehe oben
        **  IOBJTAG_DrMd,    DRMD_Jam2,
        **  IOBJTAG_BGPen,   dri->dri_Pens[SCRPEN_Background],
        */
        GRPGADTAG_BorLeft,        4,
        GRPGADTAG_BorTop,         4,
        GRPGADTAG_BorRight,       4,
        GRPGADTAG_BorBottom,      4,
        GRPGADTAG_AdjustLables,   TRUE,
        GRPGADTAG_TstAddGadget,   gb_PopupGad,
        GRPGADTAG_TstAddGadget,   gb_SliderGad,
        GRPGADTAG_TstAddGadget,   pOS_NewIObject( NULL,
            dri->dri_Names[SCRNAM_GGroupClass], 0,
            ICLTAG_DrawInfo,          dri,
            ICLTAG_Gwk,               2,
            GRPGADTAG_Horiz,          TRUE,
            GRPGADTAG_SpaceHoriz,     4,
            GRPGADTAG_TstAddGadget,   pOS_NewIObject( NULL,
                dri->dri_Names[SCRNAM_GGroupClass], 0,
                ICLTAG_DrawInfo,          dri,
                GRPGADTAG_TstAddGadget,   gb_TextGad,
                GRPGADTAG_TstAddGadget,   gb_MXGad,
                TAG_DONE),
            GRPGADTAG_TstAddGadget,   gb_ListviewGad,
            TAG_DONE),
        TAG_DONE),

    GRPGADTAG_TstAddGadget,   gb_InfotextGad,
    TAG_DONE);

  /*
  ** SAS braucht hier ein casting von TRUE/FALSE auf BOOL
  ** Alternativ knnen auch zwei Returns verwendet werden.
  */
#ifdef __SASC
  if( gb_AllGadgetGrp ) return( TRUE  );
  else                  return( FALSE );
#else
  return( gb_AllGadgetGrp ? TRUE : FALSE );
#endif
}

/******************************************************************************/

void SetDatas(struct pOS_Window *win, UBYTE jz, UWORD gm)
{
  /*
  ** setzt die gewnschte Jahreszeit in allen Gadgets und Men
  ** 'jz' ist vom Typ 'enum JahresZeit' und 'gm' vom Typ 'enum
  ** GadgetMask', kann aber Compiler-Errors verursachen, wenn der
  ** Wert z.B. als ULONG bergeben wird (bzw. SAS kann keine
  ** Enum-Datentypen verarbeiten).
  ** Die in GadgetMask (gm) enthaltenen Gadgets werden nicht verndert
  ** (haben normalerweise die Aktion veranlat)
  ** Problematik: wird zum Beispiel im ListView "gefahren", wrde
  ** ein alter Wert gesetzt, obwohl vielleicht schon ein anderer
  ** Eintrag ausgewhlt wurde!
  */

  /* String */
  if(!(gm & GM_String))
    pOS_SetGadgetAttrs(win,gb_StringGad,   STRGADTAG_String,gb_MXEntry[jz], TAG_DONE);

  /* Popup */
  if(!(gm & GM_Popup))
    pOS_SetGadgetAttrs(win,gb_PopupGad,    LVGADTAG_ItemID,jz, TAG_DONE);

  /* Slider */
  if(!(gm & GM_Slider))
    pOS_SetGadgetAttrs(win,gb_SliderGad,   PRGADTAG_Top,jz, TAG_DONE);

  /* MX */
  if(!(gm & GM_MX))
    pOS_SetGadgetAttrs(win,gb_MXGad,       MXGADTAG_Active,jz, TAG_DONE);

  /* Listview */
  if(!(gm & GM_Listview))
    pOS_SetGadgetAttrs(win,gb_ListviewGad, LVGADTAG_ItemID,jz, TAG_DONE);

  /* Infofeld (kann immer gesetzt werden) */
  pOS_SetGadgetAttrs(win,gb_InfotextGad, IOBJTAG_StaticString,gb_MXEntry[jz], TAG_DONE);
}

/******************************************************************************/

SLONG MessageLoop(struct pOS_Window *win)
{
  /*
  ** Nachrichtenschleife:
  ** pOS-Programme sollten sich immer (und jederzeit) mit CTRL-C von der Shell aus
  ** abbrechen lassen.
  **
  ** Mittles pOS_WaitSignal() wird auf das Eintreffen einer Nachricht gewartet.
  ** Die Nachrichten werden mittels pOS_GetMsg() besorgt. Wurde die Nachricht
  ** nicht vom System verarbeitet (mittels pOS_SysIMessage()), dann selber
  ** versuchen und die Nachricht am Ende mittels pOS_ReplyMsg() beantworten.
  */

  SLONG res=DOSFAIL_OK;         /* Returncode fr Programmende verwendet */
  struct pOS_IntuiMessage *msg;
  BOOL ende=FALSE;

  while(!ende)
  {
    ULONG sigs;

    sigs=pOS_WaitSignal( (1L << win->win_UserPort->mp_SigBit) | DOSSIGF_CTRL_C);
    if(sigs & DOSSIGF_CTRL_C) { res=DOSFAIL_ABORT; ende=TRUE; break; }

    while((msg=(struct pOS_IntuiMessage *) pOS_GetMsg(win->win_UserPort)))
    {
      /*
      ** Nachrichten erst mal dem OS zur Default-Verarbeitung "vorreichen";
      ** kehrt die Funktion mit TRUE zurck, wurde die Nachricht bereits "verdaut"
      ** und beantwortet. Wir drfen die Nachricht dann nicht mehr weiter behandeln,
      ** Daten auslesen oder sie zurcksenden !
      */
      if(pOS_SysIMessage(msg))
      {
        msg=NULL;
      }
      else
      {
        /* ACHTUNG: gad enthlt nur innerhalb von Gadget-IDCMPs die Gadget-Adresse */
        struct pOS_Gadget *gad=(struct pOS_Gadget *)msg->im_IAddress;
        ULONG var=gb_AktiveJahreszeit;
        enum GadgetMask gm=GM_None;

        switch(msg->im_Class)
        {
          case IDCMP_CloseWindow:
               ende=TRUE; break;


          case IDCMP_GadgetDown:      /* ICLTAG_Immediate */
          case IDCMP_GadgetUp:        /* ICLTAG_RelVerify */
          case IDCMP_GadgetAbort:     /* z.B. Slider Abbruch mit rechter Maustaste */
          case IDCMP_UpdateGadget:    /* z.B. Listview/Popup beim Verschieben */

               /*
              ** normalerweise ist es nicht notwendig die einzelnen Auswahlarten
               ** (z.B. Gadget geklickt oder losgelassen) zu unterscheiden. Deshalb
               ** erfolgt die Auswertung fr alle vier Arten gleich.
               ** die Code-Feld wird gadgetabhngig die "Unteraktion" mitangegeben.
               ** Nur fr uns interessante Aktionen sollten ausgewertet werden.
               ** So hat es z.B. keinen Einflu, wenn ein Gadget seinen Fokus verliert.
               ** Dafr sollte aber jede Auswahl (z.B. bei aufgeklapptem Popup)
               ** sofort eine (visuelle) Reaktion veranlassen.
               */
               if(gad==gb_CheckboxGad
                          && msg->im_Code==IEGUCODE_Select)
               {
                 ULONG var;
                 pOS_GetGadgetAttr(msg->im_Window,gb_CheckboxGad,ICLTAG_Selected,(ULONG*)&var);

                 if(var)  /* eingedrckt */
                 {
                   pOS_SetGadgetAttrs(win,gb_StringGad, ICLTAG_Disabled,FALSE, TAG_DONE);
                 }
                 else  /* nicht eingedrckt */
                 {
                   pOS_SetGadgetAttrs(win,gb_StringGad, ICLTAG_Disabled,TRUE, TAG_DONE);
                 }
               }

               else if(gad==gb_ButtonGad
                          && msg->im_Code==IEGUCODE_Unknown)
               {
                 pOS_SetGadgetAttrs(win,gb_CheckboxGad, ICLTAG_Selected,TRUE, TAG_DONE);
                 pOS_SetGadgetAttrs(win,gb_StringGad, ICLTAG_Disabled,FALSE, TAG_DONE);
                 gb_AktiveJahreszeit=JZ_Fruehling;
                 SetDatas(win,gb_AktiveJahreszeit,GM_None);
               }

               else if(gad==gb_StringGad
                          &&  (msg->im_Code==IEGUCODE_Ok       /* Return */
                            || msg->im_Code==IEGUCODE_Inactive /* nicht mehr aktiv */
                            || msg->im_Code==IEGUCODE_Drop))   /* String wurde "reingedropt" */
               {
                 CHAR *var2;
                 UBYTE i;
                 pOS_GetGadgetAttr(msg->im_Window,gb_StringGad,STRGADTAG_String,(ULONG*)&var2);
                 for(i=0; gb_MXEntry[i]; i++)
                 {
                   /*
                   ** Unbedingt die pOS-String-Compare Funktionen verwenden,
                   ** da nur hierdurch eine korrekte Localeuntersttzung
                   ** mglich ist.
                   */
                   if(pOS_Stricmp(var2,gb_MXEntry[i])==0)
                   {
                     var=i;  /* Trick: den Zahlenwert statt dem String einsetzen */
                     /*
                     ** GadgetMask nicht auf String setzen, damit der Text in "korrekter"
                     ** Schreibweise in das Gadget gesetzt wird. Damit aber die nachfolgende
                     ** Abfrage weis, da eine Gadgetvernderung vorliegt, setzen wir ein
                     ** "Dummy"-Gadget als ausgewhltes.
                     */
                     gm=GM_Text;
                     break;
                   }
                 }

                 if(i==JZ_MaxEntry)
                 {
                   /* Fehler falsche Eingabe: Warnton ausgeben und String-Gadget wieder aktivieren */
                   pOS_DisplayBeep(win->win_Screen,BEEPTYP_Warn);
                   pOS_ActivateGadget(win,gb_StringGad);
                 }
               }

               else if(gad==gb_PopupGad
                          &&  (msg->im_Code==IEGUCODE_UpSelect
                            || msg->im_Code==IEGUCODE_Select))
               {
                 pOS_GetGadgetAttr(msg->im_Window,gb_PopupGad,LVGADTAG_ItemID,(ULONG*)&var);
                 gm=GM_Popup;
               }

               else if(gad==gb_SliderGad
                          && msg->im_Code==IEGUCODE_Scroll)
               {
                 pOS_GetGadgetAttr(msg->im_Window,gb_SliderGad,PRGADTAG_Top,(ULONG*)&var);
                 gm=GM_Slider;
               }

               else if(gad==gb_MXGad
                          && msg->im_Code==IEGUCODE_Select)
               {
                 pOS_GetGadgetAttr(msg->im_Window,gb_MXGad,MXGADTAG_Active,(ULONG*)&var);
                 gm=GM_MX;
               }

               else if(gad==gb_ListviewGad
                          &&  (msg->im_Code==IEGUCODE_UpSelect
                            || msg->im_Code==IEGUCODE_Select))
               {
                 pOS_GetGadgetAttr(msg->im_Window,gb_ListviewGad,LVGADTAG_ItemID,(ULONG*)&var);
                 gm=GM_Listview;
               }

               /* Ersparnis: Daten nur setzen, wenn eine Vernderung vorliegt */
               if(gm != GM_None && (enum JahresZeit)var != gb_AktiveJahreszeit)
               {
                 gb_AktiveJahreszeit=var;
                 SetDatas(win,gb_AktiveJahreszeit,gm);
               }
               break;


          case IDCMP_VanillaKey:
               /*
               ** da MX-Gadgets keine Titel erlauben, mssen wir fr den improvisierten
               ** Titel auch den Shortcut selber auswerten:
               ** Da das MX-Gadget ber TAB keinen Fokus bekommt, knnen wir nicht wie
               ** blich ber Cursor hoch/tief auswhlen. Daher wird mit 'm' ein
               ** Eintrag tiefer gewhlt, mit Shift+m ein Eintrag hher.
               */
               if(msg->im_Code=='m' || msg->im_Code=='M')
               {
                 if(msg->im_Qualifier & (IEQUALIFIER_LSHIFT | IEQUALIFIER_RSHIFT))
                 {
                   /* MX rckwrts */
                   if(gb_AktiveJahreszeit > JZ_Fruehling)
                     gb_AktiveJahreszeit--;
                   else
                     gb_AktiveJahreszeit=JZ_Winter;
                 }
                 else
                 {
                   /* MX vorwrts */
                   if(gb_AktiveJahreszeit < JZ_Winter)
                     gb_AktiveJahreszeit++;
                   else
                     gb_AktiveJahreszeit=JZ_Fruehling;
                 }
                 /*
                 ** keine GM_MX-Gadgetmaske, da Gadget von uns manuel verndert wird
                 ** und deshalb noch neu gesetz werden mu
                 */
                 SetDatas(win,gb_AktiveJahreszeit,GM_None);
               }
               break;
        }
      }

      /*
      ** Nachrichten erst nach der -vollstndigen- Bearbeitung zurcksenden !
      ** und nur wenn dies von pOS_SysIMessage() noch nicht erledigt wurde
      */
      if(msg) pOS_ReplyMsg((struct pOS_Message *)msg);
    }
  }

  return( res );
}

/******************************************************************************/

#ifdef __cplusplus
extern "C"
#endif
void main(void)
{
  ULONG res=DOSFAIL_FAIL;

  /* bergebene Argumente werden geparst */
  struct pOS_DosArgs *rda;
  if((rda=pOS_ReadDosArgs(gb_PrgTemplate,args,sizeof(args)/sizeof(ULONG),
      ARGTAG_PrgHeaderText, gb_PrgHeader,    /* kurze Programm-Beschreibung */
      ARGTAG_HelpText,      gb_PrgHelpText,  /* Hilfe-Text zu den Parametern */
      ARGTAG_PrgVerText,    gb_PrgVersion,   /* Versions-String */
      TAG_END)))
  {
    /* bentigte Libraries werden geffnet */
    if((gb_IntuiBase=(struct pOS_IntuiDevice *) pOS_OpenLibrary("pIntui.library",25)))
    {
      if((gb_GfxBase=(struct pOS_GfxBase *) pOS_OpenLibrary("pGraphics.library",25)))
      {
        if((gb_UtilityBase=(struct pOS_UtilityBase *) pOS_OpenLibrary("pUtility.library",25)))
        {
          /* Pubscreen wird gelockt, ohne Parameter wird der Default-Pubscreen verwendet */
          /* (z.Z. ist dies immer der Workbench-Screen) */
          struct pOS_Screen *scr;
          if((scr=pOS_LockPubScreen((CHAR*)args[ARG_pubscreen])))
          {
            /* alle Gadgets erzeugen */
            if(CreateAllGadgets(scr->scr_DrawInfo))
            {
              /* Fenster ffnen */
              struct pOS_Window *win;
              if((win=pOS_OpenWindow(
                    SCRTAG_Width,          300,  /* gewnschte Mindestgre */
                    SCRTAG_Height,         100,  /* beim ffnen */
                    SCRTAG_Title,          PROGNAME,
                    SCRTAG_ScreenTitle,    PROGNAME" "VERSIONR"  "COPYRIGT,
                    SCRTAG_Screen,         scr,
                    SCRTAG_Flags,          WINFLGF_SizeGadget |
                                           WINFLGF_DepthGadget |
                                           WINFLGF_CloseGadget |
                                           WINFLGF_Dragbar |
                                           WINFLGF_Activate |
                                           WINFLGF_SimpleRefresh, /* Neuzeichnen wird vom Programm bernommen */
                    SCRTAG_IDCMP,          IDCMP_CloseWindow |
                                           IDCMP_StdSysMsg,  /* setzt die gebruchlisten Flags */
                    SCRTAG_AutoSizeGadget, gb_AllGadgetGrp,  /* die groe Gadget-Gruppe */
                    SCRTAG_ZipGadget,      TRUE,
                    SCRTAG_AutoAdjust,     TRUE,  /* Gre und Lage notfalls anpassen */
                    SCRTAG_UnderMouse,     TRUE,  /* unter dem Mauspfeil ffnen */
                    SCRTAG_IGfxType,       IGFXTYP_AppWin,
                    /*
                    ** bei externer Hilfetext-Datei zu setzen
                    ** SCRTAG_HelpFile,       "HELP:IntuiDemo.hlp",
                    ** SCRTAG_HelpID,         "IntuiDemo",
                    */
                    TAG_DONE)))
              {
                /* Gadgets auf Vorgaben setzen */
                SetDatas(win,gb_AktiveJahreszeit,GM_None);

                /* Nachrichten-Behandlung */
                res=MessageLoop(win);

                /* Fenster schlieen */
                pOS_CloseWindow(win);
              }
              else pOS_Printf("FEHLER: ffnen des Fensters fehlgeschlagen.\n");

              /*
              ** DeleteAllGadgets wird automatisch gemacht, wenn
              ** Tag ICLTAG_AutoDelete in Gadgets auf TRUE gesetzt wird
              ** Da wir den Tag in der obersten Gadgetgruppe gesetzt haben,
              ** werden alle Member ebenfalls freigegeben.
              */
            }
            else pOS_Printf("FEHLER: erzeugen der Gadgets fehlgeschlagen.\n");

            pOS_UnlockPubScreen(scr);
          }
          else if(args[ARG_pubscreen]) pOS_Printf("FEHLER: Pubscreen '%s' nicht vorhanden.\n",args[ARG_pubscreen]);
               else                    pOS_Printf("FEHLER: Default-Pubscreen nicht ermittelbar.\n");

          /* die geffneten Libraries wieder schlieen */
          pOS_CloseLibrary((struct pOS_Library*)gb_UtilityBase);
        }
        else pOS_Printf("FEHLER: ffnen pUtility.library V25 fehlgeschlagen.\n");

        pOS_CloseLibrary((struct pOS_Library*)gb_GfxBase);
      }
      else pOS_Printf("FEHLER: ffnen pGraphics.library V25 fehlgeschlagen.\n");

      pOS_CloseLibrary((struct pOS_Library*)gb_IntuiBase);
    }
    else pOS_Printf("FEHLER: ffnen pIntui.library V25 fehlgeschlagen.\n");

    /* ReadArgs-Argumente freigeben */
    pOS_DeleteDosArgs(rda);
  }
  else res=DOSFAIL_WARN;
  /* kein else fr ReadDosArgs() notwendig, da die Routine selber die Fehlerursache ausgibt */


  /* Fehlercode fr die aufrufende Shell/Workbench vermerken */
  /* neuere Compiler (z.B. StormC) knnen diesen aus dem Returncode von main ermitteln */
  pOS_SetShellFail(res);
}

/******************************************************************************/

